/******************************************************
Nom : bot_admin.c

Description : Fichier pricipal de l'administration du programme

Fonctions	: 	void createData(BOT *chatbot)
				void displayData(BOT *chatbot)
				void modifyData(BOT *chatbot)
				void deleteData(BOT *chatbot)
				void moderateData()
				void displayUsers()

Ecrit par	: Willy & Julian

Modifié le 	: 12 janvier 2012

Version		: 2.1 - Problème avec la fonction moderateData() (Voir ci-après)

******************************************************/

#include "bot_admin.h"

void admin(){

/* Déclaration des variable + structure */
	BOT chatbot;
	int choix=1;
	char title[30]="Menu administrateur"; 	/* Titre du mode d'administration */
	char menuTitle[30]="Menu Principal";	/* Titre de la page d'accueil */

/* Boucle du menu principal */
	while(choix != 0){

/* Affichage pour selection des différentes options du panneau administrateur */
		choix=Menu(7, \
			title,						 \
			menuTitle,					 \
			"[ structure ] - Ajouter",	 \
			"[ structure ] - Afficher",	 \
			"[ structure ] - Modifier",	 \
			"[ structure ] - Supprimer", \
			"[ structure ] - Modérer",	 \
			"[utilisateur] - Afficher",  \
			"Quitter");

/* Lecture du choix de l'utilisateur */
		switch(choix){
			case 6 :
				createData(&chatbot);
				break;
			case 5 :
				displayData(&chatbot);
				break;
			case 4 :
				modifyData(&chatbot);
				break;
			case 3 :
				deleteData(&chatbot);
				break;
			case 2 :
				moderateData();
				break;
			case 1 :
				displayUser();
				break;
			case 0 :
				ClearWithHeader("Merci d'avoir utilisé ce logiciel");
				TurnOffBold();
				Couleur("0");
				break;
	
			default :
				printf("\n\n ***Erreur Lors du choix du menu ***\n\n");
		}
	}
		

	return;
}

/* Fonction d'affichage des structures d'un fichier-theme */ 
void displayData(BOT *cb){	

	int choice, posStruct, nbRead;
	char title[]="Affichage d'une structure";
	char theme[30];
	FILE *file = NULL;

	printf("Choisissez Votre thème : \n");
	
	/* Choix du fichier-theme à lire */
	choice = Menu(4,\
		title, 		\
		"Thème de la structure", \
		"Modération", \
		"Système",	\
		"Internet",	\
		"Autres");

	switch(choice){
		case 0:
			strcpy(theme, "others.dat");
			break;
		case 1:
			strcpy(theme, "internet.dat");
			break;
		case 2:
			strcpy(theme, "system.dat");
			break;
		case 3:
			strcpy(theme, "moderate.dat");
			break;
		default:
			printf("ERR : Le thème sélectionné n'est pas encore implémenté");
	}

/* On ouvre le fichier sélectionné */
	if(file == NULL){ file = fopen(theme,"rb"); }
	else{ 
		fclose(file); 
		file = fopen(theme, "rb"); 
	}

/* Positionnement au début du fichier pour la lecture */
	posStruct = SEEK_SET;		

	do{
		ClearWithHeader(title);
		/* Affichage d'une structure */
		nbRead = readStruct(cb, file, posStruct);
		/* Si on répond 'non' ou qu'il n'y a plus de structure, on arrete de lire le fichier */
		if( nbRead == 0 || yesNoQuestion("Afficher une autre structure ?", 0) ){ break; }
		posStruct += sizeof(BOT);
	}while( nbRead > 0 );
	
	if(file != NULL){ 
		fclose(file); 	
	}
}

/* Fonction de modification des champs d'une structure */
void modifyData(BOT *cb){

	int i, choice, nbValidStruct; 
		/* Respectivement :
			Nombre de filtres
			Nombre d'octets lu (contrôle)
			Nombre de structure valides
			Nombre de filtres valides dans une structure
			Nombre de structures (obsolète)
			Tableau de position des structures valides (en octet)
		*/
	int posStruct[5]={SEEK_SET, SEEK_SET, SEEK_SET, SEEK_SET, SEEK_SET};
	char title[]="Modification de structure", theme[30];
	FILE *file = NULL;

/* Choix du fichier de recherche */
	choice=Menu(3,\
		title, 		\
		"Thème de la structure", \
		"Système",	\
		"Internet",	\
		"Autres");

	switch(choice){
		case 0:
			strcpy(theme, "others.dat");
			break;
		case 1:
			strcpy(theme, "internet.dat");
			break;
		case 2:
			strcpy(theme, "system.dat");
			break;
		default:
			printf("ERR : Le thème sélectionné n'est pas encore implémenté");
	}

		/* Ouverture du fichier lié au thème */
	file = fopen(theme,"r+b");

	/* Si le fichier n'existe pas, on retourne au menu principal */
	if(file == NULL){ 
		Menu(1, title, "Fichier du theme introuvable", "Menu principal");
		return;
	}

	/* 
		Appel de la fonction de recherche des mot-clés d'une structure 
		Cette fonction retourne le nombre de structure correspondantes
	*/

	nbValidStruct = searchStruct(cb, file, posStruct);

	/* Si des structures correspondent, on les lis */
	if(nbValidStruct > 0){
		for(i=0; i < nbValidStruct; i++){
			ClearWithHeader(title);
			Couleur("34");
			printf("%d structure(s) correspond(ent) au(x) critère(s) recherché(s)\n\n", nbValidStruct);	
			Couleur("0");
			readStruct(cb, file, posStruct[i]);	

			/* Proposition de modification de la structure */
			if( yesNoQuestion("Modifier la structure ?", 1) ){
				writeStruct(cb, file, posStruct[i]);
			}
		}
	}
	else{
		/* Si aucune ne correspond, on retourne au menu principal */
		Menu(1, title, "Aucune structure valide, retour au menu", "Menu principal");
	}
	/* Fermeture du fichier */
	if(file != NULL){ fclose(file); }
}

/* Fonction de création d'une structure */
void createData(BOT *cb){

	char title[]="Ajout de structure";
	char theme[30];

	printf("Choisissez Votre thème : \n");
	
	/* Choix du thème concerné */
	cb->theme = Menu(4,\
		title, 		\
		"Thème de la structure", \
		"Moderate", \
		"Système",	\
		"Internet",	\
		"Autres");
	
	switch(cb->theme){
		case 0:
			strcpy(theme, "others");
			break;
		case 1:
			strcpy(theme, "internet");
			break;
		case 2:
			strcpy(theme, "system");
			break;
		case 3:
			strcpy(theme, "moderate");
			break;
		default:
			strcpy(theme, "others");
	}
		
	/* Définition de l'action liée à la structure */
	do{
		cb->action= (Menu(5,\
			title,
			"Action à effectuer en cas d'utilisation de la structure", \
			"Recherche GOOGLE", \
			"Calculatrice", \
			"Ouvrir une page web", \
			"Continuer", \
			"Quitter le programme") - 1);
	
		printf("Action : %d \n\n", cb->action);
		cb->nbKeyWord = 0;
	
		/* Saisie des mots-clés & de la réponse à fournir */
		TurnOnBold();
		Couleur("34");
		printf("Entrez une chaîne de caractères contenant les mot-clés à ajouter\n");
		printf("Séparez les mot-clés avec des espaces \n\t> ");
		TurnOffBold();
		Couleur("0");
		ScanKeyWord(cb, UnlimitedScanKey(), DEL_DOUBLE);
						

		printf("Réponse à fournir : ");
		strcpy(cb->answer, UnlimitedScanKey());

		AddInFile(cb, theme);

	/* ... jusqu'à ce que l'utilisateur décide d'arrêter (choix : non) */
	}while( yesNoQuestion("Ajouter une autre structure dans ce theme ?", 1) );
}

/* Fonction de modération du fichier utilisateur */
/*
	Cette fonction pose problème dans l'exécution du programme. Elle génère une erreur
	de malloc (apparement) lors d'un fopen (désigné ci-après).

	L'erreur se produit UNIQUEMENT lorsque la fonction de modération est la première fonction 
	utilisée à l'ouverture du programme. Elle apparaît lorsque l'on choisit l'action liée
	à une structure que l'on souhaite modifier.
	Typiquement :
		> Lancement du programme
		> Fonction de modération
		> Modifier : OUI
		> choix du thème
		> choix de l'action
			*** glibc detected *** ./chatbot: malloc(): memory corruption 0x........ ***
			====== Backtrace: ======
			....
			..
			.

	Pour éviter de rencontrer l'erreur, il suffit simplement d'utiliser un autre fonction (par exemple l'affichage d'un fichier, pour pouvoir utiliser ensuite la fonction de modération.
	(Cependant, cette technique ne marche qu'alèatoirement ...)
*/
void moderateData(){
	
	BOT *tabBot = NULL;
	char theme[30] = "";
	char title[] = "Contrôle de contenu utilisateur";
	FILE *file = NULL;
	FILE *newFile = NULL;
	int i, posStruct, nbStruct;
	
	/* Ouverture du fichier de modération */
	file = fopen("moderate.dat", "rb");

	/* On se place au début du fichier ... */
	fseek(file, 0, SEEK_END);
	nbStruct = ftell(file);

	if(tabBot == NULL){ tabBot = (BOT *) malloc(nbStruct); }
	else{ tabBot = realloc(tabBot, nbStruct); }
	
	/* Et on stocke le nombre de structures dans une variable */
	nbStruct /= sizeof(BOT);

	/* S'il est vide, on retourne au menu principal */
	if(nbStruct == 0){
		Menu(1, title, "Le fichier de modération est vide", "Menu principal");
		return;
	}

	/* On se place au début du fichier */
	rewind(file);

	/* 	On stocke toutes les structures du fichier dans un tableau 
		(Permet la suppression des structures par la suite) */
	for( posStruct = 0; posStruct < nbStruct; posStruct++){ 
		fread( &tabBot[posStruct], sizeof(BOT), 1, file); 
	}

	/* Puis on ferme le fichier de modération */
	if(file != NULL){ fclose(file); }

	/* Pour chaque structure présente ... */
	for(posStruct = 0; posStruct < nbStruct; posStruct++){
	
		/* Affichage des champs de la structure */
		ClearWithHeader(title);
		printf("Structure n°%d/%d \n", (posStruct+1), nbStruct);

		printf("Nombre de mot-clés : %d \n", tabBot[posStruct].nbKeyWord);
		printf("Thème :              %d \n", tabBot[posStruct].theme);
		printf("Action :             %d \n\n", tabBot[posStruct].action);

		for(i=0; i < tabBot[posStruct].nbKeyWord; i++){ printf("Mot-clé n°%d : %s \n", i, tabBot[posStruct].keyWord[i]); }

		TurnOnBold();
		printf("\nConversation type : \n");
		TurnOffBold();
		printf("\nUtilisateur : ");
		for(i=0; i < tabBot[posStruct].nbKeyWord; i++){ printf("%s ", tabBot[posStruct].keyWord[i]); }
		printf("\nRéponse : %s \n", tabBot[posStruct].answer);
		TurnOffBold();

		/* Proposition de modification */
		if( (yesNoQuestion("Modifier la structure ?", 1)) ){ 
			/* Choix du thème dan lequel ajouter la structure */
			tabBot[posStruct].theme = Menu(3, \
				title, \
				"Theme de la structure", \
				"Système", \
				"Internet", \
				"Autres");
		
			switch(tabBot[posStruct].theme){
				case 0:
					strcpy(theme, "others.dat");
					break;
				case 1:
					strcpy(theme, "internet.dat");
					break;
				case 2:
					strcpy(theme, "system.dat");
					break;
				default:
					strcpy(theme, "others.dat");
			}

			/* Choix de l'action liée à la structure */
			tabBot[posStruct].action = (Menu(5, title, "Action liée à la structure :", \
														"Recherche GOOGLE", \
														"Calculatrice", \
														"Ouvrir une page web", \
														"Continuer", \
														"Quitter le programme") - 1);

			
			
			if(newFile == NULL){
					/* fonction fopen() causant l'erreur dans la fonction. */ 
				//	printf("\n ERROR \n"); 
					/* le printf() de la ligne au dessus s'affiche, mais ... */
				newFile = fopen(theme, "a+b");
				/*	... celui de la ligne au dessous ne s'affiche pas. */
				//	printf("\n ERROR \n"); 
				/* L'erreur réside donc bien dans le fopen() */
				fwrite(&tabBot[posStruct], sizeof(BOT), 1, newFile);
				fclose(newFile);
			}

			/* S'il y a au moins 2 structures dans le fichier... */
			if(nbStruct > 1){ 
				/* Copie de la dernière structure du fichier dans la structure actuelle */
				structCopy(&tabBot[posStruct], &tabBot[nbStruct-1]); 
			}

			/* Décrémentagitatitionnage (j'aime pas ce mot)  de la position et du nombre de structure pour ne pas oublier celle qu'on vient de déplacer */
			posStruct--;
			nbStruct--;
		}
		else if(yesNoQuestion("Supprimer cette structure ?", 1)){
			/* Même fonctionnement que précédemment si l'utilisateur choisit de supprimer la structure */
			structCopy(&tabBot[posStruct], &tabBot[nbStruct-1]);

			posStruct--;
			nbStruct--;
		}
	}

	/* 
		Une fois la modération terminée, on réécrit le tableau dans le fichier modération, 
		afin de pouvoir reprendre la modération ultérieurement 
	*/
	file = fopen("moderate.dat", "wb");
		for(posStruct = 0; posStruct < nbStruct; posStruct++){
			printf("%d structure(s) restante(s)\n", nbStruct);
			fwrite(&tabBot[posStruct], sizeof(BOT), 1, file); 
			Pause();
		}
	if(file != NULL){ fclose(file); }
}

/* Fonction de suppression d'une structure */
void deleteData(){

	BOT *tabBot = NULL;
	char theme[30];
	char title[] = "Suppression d'une structure";
	FILE *file = NULL;
	int i, posStruct, nbStruct;

	i = Menu(3, title, \
		"Theme de la structure", \
		"Système", \
		"Internet", \
		"Autres");

	switch(i){
		case 0:
			strcpy(theme, "others.dat");
			break;
		case 1:
			strcpy(theme, "internet.dat");
			break;
		case 2:
			strcpy(theme, "system.dat");
			break;
		default:
			strcpy(theme, "others.dat");
	}


	file = fopen(theme, "rb");

	fseek(file, 0, SEEK_END);
	nbStruct = ftell(file);

	if(tabBot == NULL){ tabBot = (BOT *) malloc(nbStruct); }
	else{ tabBot = realloc(tabBot, nbStruct); }

	nbStruct /= sizeof(BOT);

	if(nbStruct == 0){
		Menu(1, title, "Le fichier sélectioné est vide", "Menu principal");
		return;
	}

	rewind(file);

	for( posStruct = 0; posStruct < nbStruct; posStruct++){ 
		fread( &tabBot[posStruct], sizeof(BOT), 1, file); 
	}

	fclose(file);

	for(posStruct = 0; posStruct < nbStruct; posStruct++){
	
		ClearWithHeader(title);
		printf("Structure n°%d/%d \n", (posStruct+1), nbStruct);

		printf("Nombre de mot-clés : %d \n", tabBot[posStruct].nbKeyWord);
		printf("Thème :              %d \n", tabBot[posStruct].theme);
		printf("Action :             %d \n\n", tabBot[posStruct].action);

		for(i=0; i < tabBot[posStruct].nbKeyWord; i++){ printf("Mot-clé n°%d : %s \n", i, tabBot[posStruct].keyWord[i]); }

		TurnOnBold();
		printf("\nConversation type : \n");
		TurnOffBold();
		printf("\nUtilisateur : ");
		for(i=0; i < tabBot[posStruct].nbKeyWord; i++){ printf("%s ", tabBot[posStruct].keyWord[i]); }
		printf("\nRéponse : %s \n", tabBot[posStruct].answer);
		TurnOffBold();

		if(yesNoQuestion("Supprimer cette structure ?", 1) ){
			structCopy( &tabBot[posStruct], &tabBot[nbStruct-1]);

			posStruct--;
			nbStruct--;

			file = fopen(theme, "wb");
			for(posStruct = 0; posStruct < nbStruct; posStruct++){
				fwrite(&tabBot[posStruct], sizeof(BOT), 1, file); 
			}
			if(file != NULL){ fclose(file); }
		}
	}
}
