/************************************************************************
Nom : bot_users.c

Description : Gestion des utilisateurs

Fonctions 	: char UserRegistration(FILE *LogFile);
			  void WriteLog(FILE **LogFile,const char *format, ...);
	
Ecrit par 	: Willy & Julian

Modifié le 	: 20 décembre 2011

Version 	: 1.0 - Stable 			  
*************************************************************************/

#include "bot_users.h"

/************************************************************************
Nom 		: WriteLog

Paramètres	: LogFile - Pointeur de type FILE vers le fichier ou on 
			  souhaite écrire.
			  format - Chaine de caractère a écrire
			  
Description	: Cette fonction écrit dans le fichier pointé par LogFile
			  la chaine de caractère passé en argument, si celui-ci
			  existe. 
			  Cette fonction est un équivalent de fprintf() avec une
			  sécurité pour empecher la tentative d'écriture dans un
			  fichier inexistant.
			  
Retourne 	: Cette fonction renvoie le nombre d'octets écrit ou -1
			  si le fichier n'a pu être ouvert.
			  
Voir aussi	: fprintf() 			  
*************************************************************************/
char WriteLog(FILE *LogFile,const char *format, ...){

	int ret;
    va_list arg;

    va_start(arg,format);

	if(LogFile != NULL)
		ret = vfprintf(LogFile, format, arg);

	else
		ret = -1;

	va_end(arg);

    return ret;	
}

/************************************************************************
Nom 		: UserRegistration

Paramètres	: LogFile - Pointeur de type FILE.
			  
Description	: Cette fonction demande à l'utilisateur son pseudo. Celui-ci
			  sera ajouté a la base user.dat après vérification de sa non
			  existance. Si ce pseudo existe déja, la base de donnée 
			  user.dat ne sera pas modifiée.
			  
			  Un fichier nom_de_l'utilisateur.txt est ouvert en mode
			  ajout en fin de fichier ("a+") ou créer s'il n'existe 
			  pas.
			  
			  le pointeur LogFile pointera vers ce fichier.
			  
Retourne 	: Cette fonction retourne -1 si une erreur s'est produite,
			  0 sinon.			  
*************************************************************************/
char UserRegistration(FILE **LogFile, char *Login){

	FILE *UserListFile;							/* Pointeur vers le fichier contenant la liste des utilisateurs */

	char FormatedLogin[24];						/* Variable contenant le pseudo de l'utilisateur formaté (Suppression des Majuscule, accents, ...)*/
	char TmpLogin[20];							/* Variable permettant de stocker les pseudos lu dans le fichier "liste des utilisateurs" */
	char UserExist=USER_NOT_EXIST;				/* Variable permettant de savoir si l'utilisateur existe initialisé a USER_NOT_EXIST */
	
	struct tm *tmptime;							/* Structure utilisée pour la lecture de l'heure et du jour */
	time_t t;									/* Variable utilisée pour la lecture de l'heure */
	char Date[100];								/* Variable contenant la date et l'heure sous forme chaine de caractère */
	
	/* Demande à l'utilisateur de son pseudo */
	
	ClearWithHeader("Log in");					/* On efface la console puis on affiche le bandeau de présentation avec le sous-titre "Log in" */
	printf("Pseudo : ");						/* Affichage à l'écran du texte : "Pseudo : " */
	scanf(" %19s",Login);						/* Saisie du pseudo avec enregistrement des 20 premiers caractères dans Login */
	while(getchar() != '\n');					/* On vide le buffer d'entrée */
	
	/* Pseudo enregistré dans la variable */
	
	strcpy(FormatedLogin,Login);	/* On copie le pseudo dans la variable FormatedLogin */
	FormatChar(FormatedLogin);		/* On formate la chaine de caractère */

	/* Tester si l'utilisateur existe déja (Explication à la fin du fichier)*/
	UserListFile = fopen("user.dat","a+b");		/* Ouverture du fichier user.dat en lecture/écriture ou création si celui-ci n'existe pas */
	fseek(UserListFile,0,SEEK_SET);				/* Positionnement au début du fichier */
	
	while(fread(TmpLogin,20,1,UserListFile)>0 && UserExist == 0){ 	/* Lecture de la liste des utilisateur, chaque pseudo est stocké succéssivement dans TmpLogin */
		if(!strcmp(TmpLogin,FormatedLogin))							/* Comparaison de TmpLogin avec le pseudo entré par l'utilisateur */
			UserExist = 1;											/* Enregistrement de l'existence de l'utilisateur dans la variable UserExist */
	}
	if(UserExist == USER_NOT_EXIST){			/* Teste la non-existance de l'utilisateur */
		fseek(UserListFile,0,SEEK_END);			/* On se place à la fin du fichier user.dat */
		fwrite(FormatedLogin,20,1,UserListFile);	/* On enregistre l'utilisateur en fin de fichier */
	}
	
	strcat(FormatedLogin,".txt");				/* Ajout de l'extension au pseudo de l'utilisateur pour la création du fichier log */
	*LogFile = fopen(FormatedLogin,"a+");			/* Ouverture du fichier de log */
	
	if(*LogFile == NULL){
		ClearWithHeader("ERROR");
		printf("Une erreur est survenue lors de l'ouverture ou création du fichier log ! \nIls ne seront par conséquence pas enregistrés");
		Pause();
		return -1;
	}
	
	else{
		/* Lecture de la date et de l'heure */
		t = time(NULL);								/* Initialisation de la vatiable t*/
		tmptime = localtime(&t);					/* Initialisation de la variable tmptime contenant les données heure/date */
		if (tmptime == NULL) {						/* Teste si une erreur est survenue lors de la lecture */
			strcpy(Date,"Error date read");			/* Enregistrement de "Error date read" dans la variable Data */
		}
		else{										/* Sinon */
			strftime(Date,sizeof(Date),"%A, %d %B of %Y - %T (UTC%z)",tmptime);	/* Enregistrement des information date/heure dans Data */
		}
		
		/* Insertion d'un bandeau contenant la date et l'heure en début de log */
		fprintf(*LogFile,"\n\n*****************************************************************************\
					 \n%s \n*****************************************************************************\n",Date);
	}

	return 0;
}

void displayUser(){
	
	FILE *userFile;
	char user[20], useLessStr[30];
	int idUser, nbUser;

	userFile = fopen("user.dat", "rb");

	idUser = 0;
	
	ClearWithHeader("Affichage des utilisateurs");

	fseek(userFile, 0, SEEK_END);
	nbUser = (ftell(userFile) / 20);
	if(nbUser > 0){	
	printf("Il y a %d utilisateurs enregistrés\n", nbUser);
	if( yesNoQuestion("Souhaitez vous en afficher la liste ?", 1) ){

		ClearWithHeader("Affichage des utilisateurs");
		Couleur("34");
		TurnOnBold();
		printf("Liste des utilisateurs enregistrés\n\n");
		TurnOffBold();
		Couleur("0");

		rewind(userFile);
	
		while(fread(user, 20, 1, userFile) > 0){
			idUser++;
			printf("\t> [%d] %s \n", idUser, user);
			fseek(userFile, 20*idUser, SEEK_SET);
		}
	}

	Couleur("34");
	TurnOnBold();
	printf("\n\nLogs à afficher [nom utilisateur] :\n\t> ");
	TurnOffBold();
	Couleur("0");

	scanf("%20s", user);
	ClearBuff();

	strcat(useLessStr, "gedit ");
	strcat(useLessStr, user);
	strcat(useLessStr, ".txt");

	Clear();
	system(useLessStr);
	}
	else{ Menu(1, "Affichage des utilisateurs", "Aucun utilisateur enregistré", "Menu principal"); }
	
	fclose(userFile);
}


void addUser(){
	
	FILE *file = NULL;
	char user[20];

	file = fopen("user.dat", "ab");

	ClearWithHeader("Ajouter un utilisateur");
	
	Couleur("34");
	TurnOnBold();
	printf("Login : ");	
	TurnOffBold();
	Couleur("0");

	scanf(" %19s", user);
	ClearBuff();

	fwrite(user, 20, 1, file);

	fclose(file);
}
